@@ -16,19 +16,28 @@
 import gtk
 import gobject
 
-from os.path import join, exists
+import os
 
+import urlparse
+import urllib
+
+from gazpacho.loader.widgettree import GladeParseError
 from gazpacho import palette, editor, project, catalog
 from gazpacho.placeholder import Placeholder
 from gazpacho.path import pixmaps_dir
 from gazpacho.command import CommandManager, CommandStackView
-from gazpacho.clipboard import Clipboard, ClipboardView
+from gazpacho import clipboard
 from gazpacho.gaction import GActionsView, GAction, GActionGroup
 from gazpacho.gaction import GActionDialog, GActionGroupDialog
 from gazpacho.widget import Widget, get_widget_from_gtk_widget
-from gazpacho.l10n import _
+from gazpacho import util
 
 class Application(object):
+
+    # DND information
+    TARGET_TYPE_URI = 100
+    targets = [('text/uri-list', 0, TARGET_TYPE_URI)]
+    
     def __init__(self):
         # see catalog.py
         self._catalogs = catalog.load_all()
@@ -45,7 +54,7 @@ class Application(object):
         self._command_stack_window = None
         self._clipboard_window = None
 
-        self._clipboard = Clipboard()
+        self._clipboard = clipboard.Clipboard()
         
         # here we put views that should update when changing the project
         self._project_views = []
@@ -63,7 +72,7 @@ class Application(object):
         view = WidgetTreeView(self)
         self._project_views.insert(0, view)
         view.set_project(self._project)
-        view.set_size_request(150, 300)
+        view.set_size_request(150, 160)
         view.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
         return view
  
@@ -74,18 +83,12 @@ class Application(object):
         view.set_size_request(300, 200)
         view.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
         return view
-        
-    def _clipboard_view_create(self):
-        view = ClipboardView(self._clipboard)
-        view.set_size_request(300, 200)
-        view.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        return view
-    
+
     def _gactions_view_create(self):
         view = GActionsView()
         view.set_project(self._project)
         self._project_views.insert(0, view)        
-        view.set_size_request(150, -1)
+        view.set_size_request(150, 160)
         view.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
         return view
     
@@ -93,8 +96,9 @@ class Application(object):
         application_window = gtk.Window(gtk.WINDOW_TOPLEVEL)
         application_window.move(0, 0)
         application_window.set_default_size(700, -1)
-        gtk.window_set_default_icon_from_file(join(pixmaps_dir,
-                                                   'gazpacho-icon.png'))
+        iconfilename = os.path.join(pixmaps_dir, 'gazpacho-icon.png')
+        gtk.window_set_default_icon_from_file(iconfilename)
+                                                
         application_window.connect('delete-event', self._delete_event)
 
         # Create the different widgets
@@ -128,7 +132,7 @@ class Application(object):
         notebook.append_page(widget_view, gtk.Label(_('Widgets')))
         notebook.append_page(self.gactions_view, gtk.Label(_('Actions')))
 
-        vpaned.set_position(200)
+        vpaned.set_position(150)
 
         vpaned.add1(notebook)
         vpaned.add2(self._editor)
@@ -137,6 +141,17 @@ class Application(object):
         
         main_vbox.pack_end(self._statusbar, False)
 
+        # dnd doesn't seem to work with Konqueror, at least not when
+        # gtk.DEST_DEFAULT_ALL or gtk.DEST_DEFAULT_MOTION is used. If
+        # handling the drag-motion event it will work though, but it's
+        # a bit tricky.
+        application_window.drag_dest_set(gtk.DEST_DEFAULT_ALL,
+                                         Application.targets,
+                                         gtk.gdk.ACTION_COPY)
+        
+        application_window.connect('drag_data_received',
+                                   self._dnd_data_received_cb)
+        
         self.refresh_undo_and_redo()
 
         return application_window
@@ -217,7 +232,9 @@ class Application(object):
             ('ProjectMenu', None, _('_Project')),
             ('DebugMenu', None, _('_Debug')),
             ('HelpMenu', None, _('_Help')),
-            ('About', None, _('_About'), None, _('About'), self._about_cb)
+            ('About', None, _('_About'), None, _('About'), self._about_cb),
+            ('DumpData', None, _('Dump data'), '<control>M',
+              _('Dump debug data'), self._dump_data_cb)
             )
 
         toggle_actions = (
@@ -269,6 +286,8 @@ class Application(object):
     <menu action="DebugMenu">
       <menuitem action="ShowCommandStack"/>
       <menuitem action="ShowClipboard"/>
+      <separator/>
+      <menuitem action="DumpData"/>
     </menu>
     <menu action="HelpMenu">
       <menuitem action="About"/>
@@ -309,6 +328,11 @@ class Application(object):
 
         menu = self._ui_manager.get_widget('/MainMenu')
         toolbar = self._ui_manager.get_widget('/MainToolbar')
+
+        # Make sure that the project menu isn't removed if empty
+        project_action = self._ui_manager.get_action('/MainMenu/ProjectMenu/')
+        project_action.set_property('hide-if-empty', False)
+
         return (menu, toolbar)
     
     def _construct_statusbar(self):
@@ -321,11 +345,19 @@ class Application(object):
     def get_window(self): return self._window
     window = property(get_window)
     
-    def _push_statusbar_hint(self, menuitem, tip):
+    def _push_statusbar_hint(self, msg):
         self._statusbar.push(self._statusbar_menu_context_id, tip)
 
-    def _pop_statusbar_hint(self, menuitem, data=None):
+    def _pop_statusbar_hint(self):
         self._statusbar.pop(self._statusbar_menu_context_id)
+
+    def show_message(self, msg, howlong=5000):
+        self._statusbar.push(self._statusbar_menu_context_id, msg)
+        def remove_msg():
+            self._statusbar.pop(self._statusbar_menu_context_id)
+            return False
+        
+        gobject.timeout_add(howlong, remove_msg)
         
     def refresh_undo_and_redo(self):
         undo_item = redo_item = None
@@ -370,12 +402,83 @@ class Application(object):
         prj = project.Project(True, self)
         self._add_project(prj)
 
-    def open_project(self, path):
-        prj = project.Project.open(path, self)
 
-        if prj is not None:
-            self._add_project(prj)
-                
+    def _confirm_open_project(self, old_project, path):
+        """Ask the user whether to reopen a project that has already
+        been loaded (and thus reverting all changes) or just switch to
+        the loaded version.
+        """
+        # If the project hasn't been modified, just switch to it.
+        if not old_project.changed:
+            self._set_project(old_project)
+            return
+        
+        submsg1 = _('The project "%s" is already loaded.') % \
+                  old_project.name
+        submsg2 = _('Do you wish to revert your changes and reload the original glade file?') + \
+                  ' ' + _('Your changes will be permanently lost if you choose to revert.')
+        msg = '<span weight="bold" size="larger">%s</span>\n\n%s\n' % \
+              (submsg1, submsg2)
+        dialog = gtk.MessageDialog(self._window, gtk.DIALOG_MODAL,
+                                    gtk.MESSAGE_WARNING, gtk.BUTTONS_NONE,
+                                    msg)
+        dialog.set_title('')
+        dialog.label.set_use_markup(True)
+        dialog.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
+                           gtk.STOCK_REVERT_TO_SAVED, gtk.RESPONSE_YES)
+        dialog.set_default_response(gtk.RESPONSE_CANCEL)
+
+        ret = dialog.run()
+        if ret == gtk.RESPONSE_YES:
+            # Close the old project
+            old_project.selection_clear(False)
+            
+            for widget in old_project.widgets:
+                widget.destroy()
+
+            self._ui_manager.remove_ui(old_project.uim_id)
+            self._projects.remove(old_project)
+
+            # Open the project again
+            prj = project.Project.open(path, self)
+            if prj is not None:
+                self._add_project(prj)
+        else:
+            # No revert - just switch to the old project.
+            self._set_project(old_project)
+            
+        dialog.destroy()
+
+    def open_project(self, path):
+        # Check if the project is loaded and ask the user what to do.
+        for prj in self._projects:
+            if prj.path and prj.path == path:
+                self._confirm_open_project(prj, path)
+                return
+        
+        try:
+            prj = project.Project.open(path, self)
+
+            if prj is not None:
+                self._add_project(prj)
+        except GladeParseError, e:
+            submsg1 = _('The project could not be loaded')
+            submsg2 = _('An error occurred while parsing the file "%s".') % \
+                      os.path.abspath(path)
+            msg = '<span weight="bold" size="larger">%s</span>\n\n%s\n' % \
+                      (submsg1, submsg2)
+            dialog = gtk.MessageDialog(self._window,
+                                    gtk.DIALOG_MODAL \
+                                    | gtk.DIALOG_DESTROY_WITH_PARENT,
+                                    gtk.MESSAGE_ERROR,
+                                    gtk.BUTTONS_OK,
+                                    msg)
+            dialog.set_title('')
+            dialog.label.set_use_markup(True)
+            dialog.run()
+            dialog.destroy()
+            self._update_sensitivity()
+        
     def _add_project(self, project):
         # if the project was previously added, don't reload
         for prj in self._projects:
@@ -389,7 +492,7 @@ class Application(object):
         project_action= gtk.Action(project.name, project.name, project.name,
                                    '')
 
-        project_action.connect('activate', self._set_project, project)
+        project_action.connect('activate', self._set_project_cb, project)
         project_ui = """
         <ui>
           <menubar name="MainMenu">
@@ -399,42 +502,55 @@ class Application(object):
           </menubar>
         </ui>
         """ % project.name
-        action_group = self._ui_manager.get_action_groups()[0]
-        action_group.add_action(project_action)
+        for action_group in self._ui_manager.get_action_groups():
+            if action_group.get_name() == 'MenuActions':
+                action_group.add_action(project_action)
+                break
         
         project.uim_id = self._ui_manager.add_ui_from_string(project_ui)
 
         # connect to the project signals so that the editor can be updated
-        project.connect('widget_name_changed', self._widget_name_changed_cb)
         project.connect('selection_changed',
                          self._project_selection_changed_cb)
-
-        # make sure the palette is sensitive
-        self._palette.set_sensitive(True)
+        project.connect('project_changed', self._project_changed_cb)
 
         self._set_project(project)
 
+        # make sure the palette and the right actions are sensitive
+        self._update_sensitivity()
+
+    def _project_changed_cb(self, project):
+        self._refresh_title()
+        self._update_save_action()
+        
+    def _set_project_cb(self, action, project):
+        self._set_project(project)
+        
     def _set_project(self, project):
+        if project is self._project:
+            return
+        
         if not project in self._projects:
             print _('Could not set project because it could not be found in the list')
             return
 
-        # clear the selection in the previous project
-        if self._project:
-            self._project.selection_clear(False)
-
         self._project = project
         self._refresh_title()
 
         for view in self._project_views:
             view.set_project(self._project)
-            
+
+        self.refresh_undo_and_redo()
+        self._update_save_action()
+
         # trigger the selection changed signal to update the editor
         self._project.selection_changed()
         
     def _refresh_title(self):
         if self._project:
-            title = 'Gazpacho - %s' % self._project.name
+            title = '%s - Gazpacho' % self._project.name
+            if self._project.changed:
+                title = '*' + title
         else:
             title = 'Gazpacho'
         self._window.set_title(title)
@@ -468,21 +584,54 @@ class Application(object):
         filechooser.set_filter(file_filter)
         if filechooser.run() == gtk.RESPONSE_OK:
             path = filechooser.get_filename()
-            if path:
+            filechooser.destroy()
+            
+            if path and os.access(path, os.R_OK):
                 self.open_project(path)
-
-        filechooser.destroy()
+            else:
+                submsg1 = _('Could not open file "%s"') % os.path.abspath(path)
+                submsg2 = _('The file "%s" could not be opened. Permission denied.') % \
+                          os.path.abspath(path)
+                msg = '<span weight="bold" size="larger">%s</span>\n\n%s\n' % \
+                      (submsg1, submsg2)
+                dialog = gtk.MessageDialog(self._window,
+                                           gtk.DIALOG_MODAL | \
+                                           gtk.DIALOG_DESTROY_WITH_PARENT,
+                                           gtk.MESSAGE_ERROR,
+                                           (gtk.BUTTONS_OK),
+                                           msg)
+                dialog.set_title('')
+                dialog.label.set_use_markup(True)
+                dialog.run()
+                dialog.destroy()
+        else:
+            filechooser.destroy()
 
     def _save(self, project, path):
         """ Internal save """
-        project.save(path)
-#        self._refresh_project_entry(project)
-        self._refresh_title()
-        
+        if not os.path.exists(path) or os.access(path, os.W_OK):
+            project.save(path)
+            #self._refresh_project_entry(project)
+            self._refresh_title()
+            success = True
+        else:
+            text = _('Could not save to file "%s"\nPermission denied.' % path)
+            msg = gtk.MessageDialog(parent=self._window,
+                                    flags=gtk.DIALOG_MODAL \
+                                    | gtk.DIALOG_DESTROY_WITH_PARENT,
+                                    type=gtk.MESSAGE_ERROR,
+                                    buttons=(gtk.BUTTONS_OK),
+                                    message_format=text)
+            msg.run()
+            msg.destroy()
+            success = False
+
+        return success
+    
     def _save_cb(self, action):
         project = self._project
         if project.path is not None:
-            project.save(project.path)
+            self._save(project, project.path)
             return
 
         # If instead we don't have a path yet, fire up a file chooser
@@ -505,30 +654,32 @@ class Application(object):
                                              gtk.RESPONSE_CANCEL,
                                              gtk.STOCK_SAVE,
                                              gtk.RESPONSE_OK))
-        filechooser.set_current_name("%s.glade" % project.name)
+        current_name = project.name
+        if not current_name.endswith(".glade"):
+            current_name = "%s.glade" % current_name
+        filechooser.set_current_name(current_name)        
         filechooser.set_default_response(gtk.RESPONSE_OK)
         while True:
             if filechooser.run() == gtk.RESPONSE_OK:
                 path = filechooser.get_filename()
-                if exists(path):
+                if os.path.exists(path):
+                    text = _('There is a file with that name already.\nWould you like to overwrite it?')
                     msg = gtk.MessageDialog(parent=self._window,
                                             flags=gtk.DIALOG_MODAL \
                                             | gtk.DIALOG_DESTROY_WITH_PARENT,
                                             type=gtk.MESSAGE_WARNING,
                                             buttons=(gtk.BUTTONS_YES_NO),
-                                            message_format=_('There is a file with that name already.\nWould you like to overwrite it?'))
+                                            message_format=text)
                     result = msg.run()
                     msg.destroy()
                     # the user want to overwrite the file
                     if result == gtk.RESPONSE_YES:
-                        self._save(project, path)
-                        saved = True
+                        saved = self._save(project, path)
                         break
 
                 # the file does not exists. we can save it without worries
                 else:
-                    self._save(project, path)
-                    saved = True
+                    saved = self._save(project, path)
                     break
             # maybe the user changed his/her opinion and don't want to
             # save (by clicking cancel on the file chooser)
@@ -539,48 +690,54 @@ class Application(object):
         return saved
 
     def _confirm_close_project(self, project):
-        submsg1 = _('Save changes to project \"%s\" before closing?') % \
-                  project.name
-        submsg2 = _("Your changes will be lost if you don't save them")
-        msg = "<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s\n" % \
-              (submsg1, submsg2)
-        dialog = gtk.MessageDialog(self._window, gtk.DIALOG_MODAL,
-                                    gtk.MESSAGE_QUESTION, gtk.BUTTONS_NONE,
-                                    msg)
-        dialog.label.set_use_markup(True)
-        dialog.add_buttons(_("Close without Saving"), gtk.RESPONSE_NO,
-                            gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
-                            gtk.STOCK_SAVE, gtk.RESPONSE_YES)
-        dialog.set_default_response(gtk.RESPONSE_YES)
-        while True:
-            ret = dialog.run()
-            if ret == gtk.RESPONSE_YES:
-                close = True
+        """Show a dialog asking the user whether or not to save the
+        project before closing them.
+
+        Return False if the user has chosen not to close the project.
+        """
+        return self._confirm_close_projects([project])
+
+    def _confirm_close_projects(self, projects):
+        """Show a dialog listing the projects and ask whether the user
+        wants to save them before closing them.
+
+        Return False if the user has chosen not to close the projects.
+        """
+        if not projects:
+            return True
+        
+        if len(projects) == 1:
+            dialog = SingleCloseConfirmationDialog(projects[0], self._window)
+        else:
+            dialog = MultipleCloseConfirmationDialog(projects, self._window)
+
+        ret = dialog.run()
+        if ret == gtk.RESPONSE_YES:
+            # Go through the chosen projects and save them. Ask for a
+            # file name if necessary.
+            close = True
+            projects_to_save = dialog.get_projects()
+            for project in projects_to_save:
                 if project.path:
                     project.save(project.path)
-                    close = True
                 else:
                     title = _('Save as')
                     saved = self._project_save_as(title, project)
+                    # If the user has pressed cancel we abort everything.
                     if not saved:
-                        continue
-            elif ret == gtk.RESPONSE_NO:
-                close = True
-            else:
-                close = False
-            break
+                        close = False
+                        break
+        elif ret == gtk.RESPONSE_NO:
+            # The user has chosen not to save any projects.
+            close = True
+        else:
+            # The user has cancel the close request.
+            close = False
+
         dialog.destroy()
         return close
-    
-    def _close_cb(self, action):
-        if not self._project:
-            return
-
-        if self._project.changed:
-            close = self._confirm_close_project(self._project)
-            if not close:
-                return
 
+    def close_current_project(self):
         self._project.selection_clear(False)
         self._project.selection_changed()
         
@@ -596,20 +753,30 @@ class Application(object):
                 view.set_project(None)
             self._project = None
             self._refresh_title()
-            self._palette.set_sensitive(False)
+            self.refresh_undo_and_redo()
+            self._update_sensitivity()
             return
 
-        # this is needed to prevent clearing the selection of a closed project
-        self._project = None
         self._set_project(self._projects[0])
+        
+    def _close_cb(self, action):
+        if not self._project:
+            return
+
+        if self._project.changed:
+            close = self._confirm_close_project(self._project)
+            if not close:
+                return
+
+        self.close_current_project()
+
 
     def _quit_cb(self, action=None):
-        projects = self._projects
-        for project in projects:
-            if project.changed:
-                if not self._confirm_close_project(project):
-                    return
-                self._projects.remove(project)
+        unsaved_projects = [p for p in self._projects if p.changed]
+        close = self._confirm_close_projects(unsaved_projects)
+        if not close:
+            return
+
         gtk.main_quit()
 
     def _undo_cb(self, action):
@@ -636,20 +803,31 @@ class Application(object):
         if selection:
             gwidget = get_widget_from_gtk_widget(selection[0])
             self._command_manager.copy(gwidget)
+            # Make sure the sensitivity of the edit actions (cut,
+            # copy,paste,delete) are updated.
+            self._update_edit_actions()
     
     def _copy_cb(self, action):
         self._real_copy()
 
     def _real_paste(self):
-        if self._clipboard.get_selected_widget() is None:
+        item = self._clipboard.get_selected_item()
+        if item is None:
             print _('There is nothing in the clipboard to paste')
             return
-        selection = self._project.selection
-        if selection and isinstance(selection[0], Placeholder):
-            self._command_manager.paste(selection[0])
+
+        if self._project is None:
+            print _("No project has been specified. Cannot paste the widget")
+            return
+        
+        selection = None
+        if self._project.selection:
+            selection = self._project.selection[0]
+
+        if isinstance(selection, Placeholder) or item.is_toplevel:
+            self._command_manager.paste(selection, self._project)
         else:
             print _('Please, select a placeholder to paste the widget on')
-            return
         
     def _paste_cb(self, action):
         self._real_paste()
@@ -666,9 +844,9 @@ class Application(object):
         dialog.set_border_width(12)
         dialog.vbox.set_spacing(6)
         image = gtk.Image()
-        image.set_from_file(join(pixmaps_dir, 'gazpacho.png'))
+        image.set_from_file(os.path.join(pixmaps_dir, 'gazpacho.png'))
 
-        version = gtk.Label('<span size="xx-large"><b>Gazpacho 0.5.2</b></span>')
+        version = gtk.Label('<span size="xx-large"><b>Gazpacho 0.5.3</b></span>')
         version.set_use_markup(True)
         description = gtk.Label(_('Gazpacho is a User Interface Builder for GTK+.\n\n')+'\n\n<small>(C) 2004,2005 SICEm S.L.</small>')
         description.set_use_markup(True)
@@ -693,9 +871,6 @@ class Application(object):
             self._palette.unselect_widget()
             self._add_class = None
 
-    def _widget_name_changed_cb(self, project, widget):
-        self._editor.update_widget_name()
-
     def _project_selection_changed_cb(self, project):
         if self._project != project:
             self._set_project(project)
@@ -710,6 +885,8 @@ class Application(object):
             else:
                 self._editor.load_widget(None)
 
+        self._update_edit_actions()
+        
     # debugging windows
     def _delete_event_for_debugging_window(self, window, event, action):
         # this will hide the window
@@ -742,20 +919,21 @@ class Application(object):
                 self._command_stack_window.show_all()
                 
     def _show_clipboard_cb(self, action):
+        """Show/hide the clipboard window."""
         if self._clipboard_window is None:
-            view = self._clipboard_view_create()
-            title = _('Clipboard contents')
             action = self._ui_manager.get_action('/MainMenu/DebugMenu/ShowClipboard')
-            self._clipboard_window = self._create_debugging_window(view,
-                                                                   title,
-                                                                   action)
-            self._clipboard_window.show()
+            self._clipboard_window = clipboard.ClipboardWindow(self.window,
+                                                               self._clipboard)
+            self._clipboard_window.connect('delete-event',
+                                           self._delete_event_for_debugging_window,
+                                           action)
+            self._clipboard_window.show_window()
         else:
             if self._clipboard_window.get_property('visible'):
-                self._clipboard_window.hide()
+                self._clipboard_window.hide_window()
             else:
-                self._clipboard_window.show_all()
-                
+                self._clipboard_window.show_window()
+
     # useful properties
     def get_add_class(self): return self._add_class
     add_class = property(get_add_class)
@@ -779,12 +957,6 @@ class Application(object):
     
     def get_clipboard(self):
         return self._clipboard
-    
-    def get_clipboard_view(self):
-        if self._clipboard_window is not None:
-            return self._clipboard_window.get_child()
-        else:
-            return None
 
     # actions and action groups
     def _real_add_action(self):
@@ -848,3 +1020,275 @@ class Application(object):
         gaction = self.gactions_view.get_selected_action()
         if gaction is not None:
             self._real_edit_action(gaction)
+
+    def _update_sensitivity(self):
+        """Update sensitivity of actions and UI elements (Palette) depending on
+        the project list. If the list is empty we should unsensitive
+        everything. Otherwise we should set sensitive to True.
+        """
+        sensitive = len(self._projects) > 0
+        
+        self._palette.set_sensitive(sensitive)
+        
+        action_paths = ['/MainMenu/FileMenu/Save',
+                       '/MainMenu/FileMenu/SaveAs',
+                       '/MainMenu/FileMenu/Close',
+                       '/MainMenu/EditMenu/Copy',
+                       '/MainMenu/EditMenu/Cut',
+                       '/MainMenu/EditMenu/Paste',
+                       '/MainMenu/EditMenu/Delete',
+                       '/MainMenu/ActionMenu/AddAction',
+                       '/MainMenu/ActionMenu/RemoveAction',
+                       '/MainMenu/ActionMenu/EditAction']
+        u_list = []
+        s_list = []
+        if sensitive:
+            s_list = action_paths
+        else:
+            u_list = action_paths
+
+        self._change_action_state(sensitive=s_list, unsensitive=u_list)
+        self._update_edit_actions()
+        self._update_save_action()
+
+    def _update_save_action(self):
+        """Update the state of the Save action."""
+        if self._project is None:
+            return # _update_sensitivity() takes care of this situation
+        
+        u_list = []
+        s_list = []        
+        if self._project.changed:
+            s_list += ['/MainMenu/FileMenu/Save',]
+        else:
+            u_list += ['/MainMenu/FileMenu/Save',]            
+        self._change_action_state(sensitive=s_list, unsensitive=u_list)
+
+    def _update_edit_actions(self):
+        """Update the state of the Cut, Copy, Paste and Delete actions."""
+        if self._project is None:
+            return # _update_sensitivity() takes care of this situation
+
+        u_list = []
+        s_list = []
+        selection = self._project.selection
+        if selection:
+            # No need to Cut, Copy or Delete a placeholder
+            if isinstance(selection[0], Placeholder):
+                u_list += ['/MainMenu/EditMenu/Copy',
+                           '/MainMenu/EditMenu/Cut']
+                # Unless it's in a Box
+                parent = util.get_parent(selection[0])
+                # XXX Not sure if we should hardcode this here
+                if parent and isinstance(parent.gtk_widget, gtk.Box):
+                    s_list.append('/MainMenu/EditMenu/Delete')
+                else:
+                    u_list.append('/MainMenu/EditMenu/Delete')
+            else:
+                s_list += ['/MainMenu/EditMenu/Copy',
+                           '/MainMenu/EditMenu/Cut',
+                           '/MainMenu/EditMenu/Delete']
+        else:
+            u_list += ['/MainMenu/EditMenu/Copy',
+                       '/MainMenu/EditMenu/Cut',
+                       '/MainMenu/EditMenu/Delete']
+
+        # Unless the widget is toplevel it can only be pasted on a placeholder
+        item = self._clipboard.get_selected_item()
+        if item and (item.is_toplevel
+                     or (selection and isinstance(selection[0],
+                                                  Placeholder))):
+            s_list.append('/MainMenu/EditMenu/Paste')
+        else:
+            u_list.append('/MainMenu/EditMenu/Paste')
+            
+        self._change_action_state(sensitive=s_list, unsensitive=u_list)
+
+    def _change_action_state(self, sensitive=[], unsensitive=[]):
+        """Set sensitive True for all the actions whose path is on the
+        sensitive list. Set sensitive False for all the actions whose path
+        is on the unsensitive list.
+        """
+        for action_path in sensitive:
+            action = self._ui_manager.get_action(action_path)
+            action.set_property('sensitive', True)
+        for action_path in unsensitive:
+            action = self._ui_manager.get_action(action_path)
+            action.set_property('sensitive', False)            
+
+    def get_current_context(self):
+        """Return the context associated with the current project or None if
+        there is not such a project.
+        """
+        if self._project is None:
+            return None
+        return self._project.context
+
+    def refresh_editor(self):
+        self._editor.refresh()
+
+    def _dnd_data_received_cb(self, widget, context, x, y, data, info, time):
+        """Callback that handles drag 'n' drop of glade files."""
+        if info == Application.TARGET_TYPE_URI:    
+            for uri in data.data.split('\r\n'):
+                uri_parts = urlparse.urlparse(uri)
+                if uri_parts[0] == 'file':                
+                    path = urllib.url2pathname(uri_parts[2])
+                    self.open_project(path)
+
+
+    def _dump_data_cb(self, action):
+        """This method is only useful for debugging.
+
+        Any developer can print whatever he/she wants here
+        the only rule is: clean everything before you commit it.
+
+        This will be called upon CONTROL+M or by using the menu
+        item Debug/Dump Data
+        """
+
+class CloseConfirmationDialog(gtk.Dialog):
+    """A base class for the close confirmation dialogs. It lets the
+    user choose to Save the projects, Close without saving them and to
+    abort the close operation.
+    """
+    
+    def __init__(self, projects, toplevel=None):
+        gtk.Dialog.__init__(self,
+                            title='',
+                            parent=toplevel,
+                            flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
+        
+        self.add_buttons(_("Close _without Saving"), gtk.RESPONSE_NO,
+                         gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
+                         gtk.STOCK_SAVE, gtk.RESPONSE_YES)
+        
+        self.set_default_response(gtk.RESPONSE_YES)
+        
+        self.set_border_width(6)
+        self.set_resizable(False)
+        self.set_has_separator(False)
+        self.vbox.set_spacing(12)
+
+        # Add the warning image to the dialog
+        warning_image = gtk.image_new_from_stock(gtk.STOCK_DIALOG_WARNING,
+                                                 gtk.ICON_SIZE_DIALOG)
+        warning_image.set_alignment(0.5, 0)
+
+        self._hbox = gtk.HBox(False, 12)        
+        self._hbox.set_border_width(6)
+        self._hbox.pack_start(warning_image, False, False, 0)
+        
+        self.vbox.pack_start(self._hbox)
+
+
+class SingleCloseConfirmationDialog(CloseConfirmationDialog):
+    """A close confirmation dialog for a single project."""
+    
+    def __init__(self, project, toplevel=None):
+        CloseConfirmationDialog.__init__(self, toplevel)
+        self._project = project
+
+        submsg1 = _('Save changes to project "%s" before closing?') % \
+                  project.name
+        submsg2 = _("Your changes will be lost if you don't save them.")
+        msg = '<span weight="bold" size="larger">%s</span>\n\n%s\n' % \
+              (submsg1, submsg2)
+        label = gtk.Label(msg)
+        label.set_use_markup(True)
+        label.set_line_wrap(True)
+        label.set_alignment(0.0, 0.5)        
+        self._hbox.pack_start(label)
+
+        self.vbox.show_all()
+
+    def get_projects(self):
+        """Get a list of the projects that should be saved."""
+        return [self._project]
+
+
+class MultipleCloseConfirmationDialog(CloseConfirmationDialog):
+    """A close confirmation dialog for a multiple project. It presents
+    a list of the projects and let the user choose which to save."""
+    
+    def __init__(self, projects, toplevel=None):
+        CloseConfirmationDialog.__init__(self, toplevel)        
+        self._projects = projects
+        self._projects.sort(lambda x, y: cmp(x.name, y.name))
+
+        # ListStore (boolean, Project) storing information about which
+        # projects should be saved.
+        self._model = None
+
+        submsg1 = ('There are %d projects with unsaved changes. Do you want to save those changes before closing?') % \
+                 len(projects)        
+        submsg2 = _("Your changes will be lost if you don't save them.")
+        msg = '<span weight="bold" size="larger">%s</span>\n\n%s\n' % \
+              (submsg1, submsg2)
+        label = gtk.Label(msg)
+        label.set_use_markup(True)
+        label.set_line_wrap(True)
+        label.set_alignment(0.0, 0.5)   
+
+        list_label = gtk.Label()
+        list_label.set_markup_with_mnemonic(_("S_elect the projects you want to save:"))
+        list_label.set_line_wrap(True)
+        list_label.set_alignment(0.0, 0.5)
+        list_label.set_padding(0, 6)
+        
+        view = self._create_project_list()
+        list_label.set_mnemonic_widget(view)
+        
+        scrolled_window = gtk.ScrolledWindow()
+        scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        scrolled_window.add_with_viewport(view)        
+        scrolled_window.set_size_request(200, 100)
+        
+        project_vbox = gtk.VBox(False)
+        project_vbox.pack_start(label)
+        project_vbox.pack_start(list_label)
+        project_vbox.pack_start(scrolled_window)
+        self._hbox.pack_start(project_vbox)
+        
+        self.vbox.show_all()
+
+    def _create_project_list(self):
+        """Create, populate and return a TreeView containing the
+        unsaved projects."""
+        # Create a ListStore model
+        self._model = gtk.ListStore(gobject.TYPE_BOOLEAN, object)
+        for project in self._projects:
+            self._model.append([True, project])
+
+        # Create the TreeView
+        view = gtk.TreeView(self._model)
+        view.set_headers_visible(False)
+
+        # Create the check-box column
+        toggle_renderer = gtk.CellRendererToggle()
+        toggle_renderer.set_property('activatable', True)
+        toggle_renderer.connect("toggled", self._toggled_cb, (self._model, 0))
+        toggle_column = gtk.TreeViewColumn('Save', toggle_renderer)
+        toggle_column.add_attribute(toggle_renderer, 'active', 0)
+        view.append_column(toggle_column)
+
+        # Create the project column
+        def render_func(treeviewcolumn, renderer, model, iter):
+            project = model.get_value(iter, 1)
+            renderer.set_property('text', project.name)
+            return
+        text_renderer = gtk.CellRendererText()        
+        text_column = gtk.TreeViewColumn('Project', text_renderer)
+        text_column.set_cell_data_func(text_renderer, render_func)
+        view.append_column(text_column)
+
+        return view
+
+    def _toggled_cb(self, renderer, path, user_data):
+        """Callback to change the state of the check-box."""
+        model, column = user_data
+        model[path][column] = not model[path][column]
+        
+    def get_projects(self):
+        """Get a list of the projects that should be saved."""        
+        return [p[1] for p in self._model if p[0]]
