@@ -1,4 +1,4 @@
-# Copyright (C) 2004,2005 by SICEm S.L. and Imendio
+# Copyright (C) 2004,2005 by SICEm S.L. and Imendio AB
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public License
@@ -16,10 +16,7 @@
 import gtk
 import gobject
 
-from gazpacho.widget import Widget, copy_widget
-from gazpacho.util import get_parent
-from gazpacho.gaction import GAction, GActionGroup
-from gazpacho.l10n import _
+from gazpacho import gaction, util, widget
 
 class CommandManager(object):
     """This class is the entry point accesing the commands.
@@ -62,45 +59,50 @@ class CommandManager(object):
     # for every possible command we have a method here
     #
 
-    def delete(self, widget):
+    def delete(self, widget_instance):
         # internal children cannot be deleted. Should we notify the user?
-        if widget.internal:
+        if widget_instance.internal_name is not None:
             return
-        description = _("Delete %s") % widget.name
-        cmd = CommandCreateDelete(widget, None, widget.get_parent(), False,
+        description = _("Delete %s") % widget_instance.name
+        cmd = CommandCreateDelete(widget_instance, None,
+                                  widget_instance.get_parent(), False,
                                   description, self.app)
         cmd.execute()
-        cmd.push_undo(widget.project)
+        cmd.push_undo(widget_instance.project)
 
-    def create(self, klass, placeholder, project, parent=None, interactive=True):
+    def create(self, klass, placeholder, project, parent=None,
+               interactive=True):
         if placeholder:
-            parent = get_parent(placeholder)
+            parent = util.get_parent(placeholder)
             if parent is None:
                 return
 
         if project is None:
             project = parent.project
 
-        widget = Widget(klass, project, interactive=interactive)
-        if widget is None:
+        widget_instance = widget.Widget(klass, project)
+        widget_instance.create_gtk_widget(interactive)
+        if widget_instance is None:
             return
 
-        description = _("Create %s") % widget.name
+        description = _("Create %s") % widget_instance.name
 
-        cmd = CommandCreateDelete(widget, placeholder, parent, True, description,
-                                  self.app)
+        cmd = CommandCreateDelete(widget_instance, placeholder, parent, True,
+                                  description, self.app)
         cmd.execute()
-        cmd.push_undo(widget.project)
+        cmd.push_undo(widget_instance.project)
 
-        self.app.palette.unselect_widget()
+        if not self.app.palette.persistent_mode:
+            self.app.palette.unselect_widget()
 
     def delete_placeholder(self, placeholder):
-        parent = get_parent(placeholder)
-        if len(parent.widget.get_children()) == 1:
+        parent = util.get_parent(placeholder)
+        if len(parent.gtk_widget.get_children()) == 1:
             return
         
         description = _("Delete placeholder")
-        cmd = CommandDeletePlaceholder(placeholder, parent, description, self.app)
+        cmd = CommandDeletePlaceholder(placeholder, parent, description,
+                                       self.app)
         cmd.execute()
         cmd.push_undo(parent.project)
         
@@ -116,6 +118,7 @@ class CommandManager(object):
         dsc = _('Setting %s of %s') % (property.klass.name, gwidget.name)
         cmd = CommandSetProperty(property, value, dsc, self.app)
         cmd.execute()
+        gwidget.project.changed = True
         cmd.push_undo(gwidget.project)
 
     def set_translatable_property(self, property, value, comment,
@@ -127,75 +130,78 @@ class CommandManager(object):
                                              has_context,
                                              dsc, self.app)
         cmd.execute()
+        gwidget.project.changed = True
         cmd.push_undo(gwidget.project)
 
-    def set_name(self, widget, name):
-        dsc = _('Renaming %s to %s') % (widget.name, name)
-        cmd = CommandSetName(widget, widget.name, name, dsc, self.app)
-        cmd.execute()
-        cmd.push_undo(widget.project)
-
-    def add_signal(self, widget, signal):
+    def add_signal(self, widget_instance, signal):
         dsc = _('Add signal handler %s') % signal['handler']
-        cmd = CommandAddRemoveSignal(True, signal, widget, dsc, self.app)
-        cmd.execute()
-        cmd.push_undo(widget.project)
-
-    def remove_signal(self, widget, signal):
-        dsc = _('Remove signal handler %s') % signal['handler']
-        cmd = CommandAddRemoveSignal(False, signal, widget, dsc, self.app)
-        cmd.execute()
-        cmd.push_undo(widget.project)
-
-    def copy(self, widget):
-        dsc = _('Copy widget %s into the clipboard') % widget.name
-        op = CommandCopyCutPaste.COPY
-        copy = copy_widget(widget)
-        cmd = CommandCopyCutPaste(copy, None, op, dsc, self.app)
-        cmd.execute()
-        cmd.push_undo(widget.project)
-        
-    def cut(self, widget):
-        dsc = _('Cut widget %s into the clipboard') % widget.name
-        op = CommandCopyCutPaste.CUT
-        cmd = CommandCopyCutPaste(widget, None, op, dsc, self.app)
-        cmd.execute()
-        cmd.push_undo(widget.project)
-        
-    def paste(self, placeholder):
-        clipboard = self.app.get_clipboard()
-        widget = clipboard.get_selected_widget()
-        dsc = _('Paste widget %s from the clipboard') % widget.name
-        op = CommandCopyCutPaste.PASTE
-        cmd = CommandCopyCutPaste(widget, placeholder, op, dsc, self.app)
-        cmd.execute()
-        cmd.push_undo(widget.project)
-
-    def add_action(self, values, parent, project):
-        gaction = GAction(parent, values['name'], values['label'],
-                          values['tooltip'], values['stock_id'],
-                          values['callback'], values['accelerator'])
-        dsc = _('Add action %s') % gaction.name
-        cmd = CommandAddRemoveAction(parent, gaction, True, dsc, self.app)
-        cmd.execute()
-        cmd.push_undo(project)
-    
-    def remove_action(self, gaction, project):
-        dsc = _('Remove action %s') % gaction.name
-        cmd = CommandAddRemoveAction(gaction.parent, gaction, False, dsc,
+        cmd = CommandAddRemoveSignal(True, signal, widget_instance, dsc,
                                      self.app)
         cmd.execute()
-        cmd.push_undo(project)
-    
-    def edit_action(self, gaction, new_values, project):
-        dsc = _('Edit action %s') % gaction.name
-        cmd = CommandEditAction(gaction, new_values, dsc, self.app)
+        cmd.push_undo(widget_instance.project)
+
+    def remove_signal(self, widget_instance, signal):
+        dsc = _('Remove signal handler %s') % signal['handler']
+        cmd = CommandAddRemoveSignal(False, signal, widget_instance, dsc,
+                                     self.app)
+        cmd.execute()
+        cmd.push_undo(widget_instance.project)
+
+    def copy(self, widget_instance):
+        """Add a copy of the widget to the clipboard.
+
+        Note that it does not make sense to undo this operation
+        """
+        clipboard = self.app.get_clipboard()
+        clipboard.add_widget(widget_instance)
+        
+    def cut(self, widget_instance):
+        dsc = _('Cut widget %s into the clipboard') % widget_instance.name
+        clipboard = self.app.get_clipboard()
+        clipboard.add_widget(widget_instance)
+        cmd = CommandCutPaste(widget_instance, widget_instance.project, None,
+                              CommandCutPaste.CUT, dsc, self.app)
+        cmd.execute()
+        cmd.push_undo(widget_instance.project)
+        
+    def paste(self, placeholder, project):
+        if project is None:
+            raise ValueError("No project has been specified. Cannot paste "
+                             "the widget")
+
+        clipboard = self.app.get_clipboard()
+        gwidget = clipboard.get_selected_widget(project)
+
+        dsc = _('Paste widget %s from the clipboard') % gwidget.name
+        cmd = CommandCutPaste(gwidget, project, placeholder,
+                              CommandCutPaste.PASTE, dsc, self.app)
         cmd.execute()
         cmd.push_undo(project)
-        project.change_action_name(gaction)
+
+    def add_action(self, values, parent, project):
+        gact = gaction.GAction(parent, values['name'], values['label'],
+                               values['tooltip'], values['stock_id'],
+                               values['callback'], values['accelerator'])
+        dsc = _('Add action %s') % gact.name
+        cmd = CommandAddRemoveAction(parent, gact, True, dsc, self.app)
+        cmd.execute()
+        cmd.push_undo(project)
+    
+    def remove_action(self, gact, project):
+        dsc = _('Remove action %s') % gact.name
+        cmd = CommandAddRemoveAction(gact.parent, gact, False, dsc, self.app)
+        cmd.execute()
+        cmd.push_undo(project)
+    
+    def edit_action(self, gact, new_values, project):
+        dsc = _('Edit action %s') % gact.name
+        cmd = CommandEditAction(gact, new_values, dsc, self.app)
+        cmd.execute()
+        cmd.push_undo(project)
+        project.change_action_name(gact)
 
     def add_action_group(self, name, project):
-        gaction_group = GActionGroup(name)
+        gaction_group = gaction.GActionGroup(name)
         dsc = _('Add action group %s') % gaction_group.name
         cmd = CommandAddRemoveActionGroup(gaction_group, project, True,
                                           dsc, self.app)
@@ -235,7 +241,7 @@ class Command(object):
     After you execute a command in the usual way you should put that
     command in the command stack of that project and that's what
     the push_undo method does. Otherwise no undo will be available.
-    
+   
     Some commands unifies themselves. This means that if you execute
     several commands of the same type one after the other, they will be
     treated as only one big command in terms of undo/redo. In other words,
@@ -306,37 +312,47 @@ class Command(object):
         self._app.refresh_undo_and_redo()
 
 class CommandCreateDelete(Command):
-    def __init__(self, widget=None, placeholder=None, parent=None, create=True,
-                 description=None, application=None):
+    def __init__(self, widget_instance=None, placeholder=None, parent=None,
+                 create=True, description=None, application=None):
         Command.__init__(self, description, application)
 
-        self._widget = widget
+        self._widget_instance = widget_instance
         self._placeholder = placeholder
         self._parent = parent
         self._create = create
         self._initial_creation = create
+        self._ui_defs = {}
         
     def _create_execute(self):
-        from gazpacho.placeholder import Placeholder
-        if not self._widget.widget.flags() & gtk.TOPLEVEL:
+        from gazpacho import placeholder
+
+        widget_instance = self._widget_instance
+
+        # if _delete_execute was called on a toolbar/menubar the uim removed
+        # its ui definitions so we need to add them again
+        if len(self._ui_defs.keys()) > 0:
+            widget_instance.project.uim.add_widget(widget_instance,
+                                                   self._ui_defs)
+            self._ui_defs = {}
+        
+        if not widget_instance.gtk_widget.flags() & gtk.TOPLEVEL:
             if self._placeholder is None:
-                for child in self._parent.widget.get_children():
-                    if isinstance(child, Placeholder):
+                for child in self._parent.gtk_widget.get_children():
+                    if isinstance(child, placeholder.Placeholder):
                         self._placeholder = child
                         break
-            self._widget.replace(self._placeholder, self._widget.widget,
-                                 self._parent)
+            widget.replace_widget(self._placeholder,
+                                  widget_instance.gtk_widget,
+                                  self._parent)
 
-        if isinstance(self._widget.widget, gtk.Widget):
-            self._widget.get_glade_property('visible').value = True
-            
-        self._widget.project.add_widget(self._widget.widget)
-        self._widget.project.selection_set(self._widget.widget, True)
+        self._widget_instance.project.add_widget(widget_instance.gtk_widget)
+        self._widget_instance.project.selection_set(widget_instance.gtk_widget,
+                                                    True)
 
-        if isinstance(self._widget.widget, gtk.Widget):
-            self._widget.widget.show_all()
+        if isinstance(widget_instance.gtk_widget, gtk.Widget):
+            widget_instance.gtk_widget.show_all()
 
-        if self._widget.is_toplevel():
+        if widget_instance.is_toplevel():
             # we have to attach the accelerators groups so key shortcuts
             # keep working when this window has the focus. Only do
             # this the first time when creating a window, not when
@@ -344,24 +360,39 @@ class CommandCreateDelete(Command):
             # set by then
             if self._initial_creation:
                 for ag in self._app.get_accel_groups():
-                    self._widget.widget.add_accel_group(ag)
+                    widget_instance.gtk_widget.add_accel_group(ag)
                 self._initial_creation = False
                 
             # make window management easier by making created windows
             # transient for the editor window
-            self._widget.widget.set_transient_for(self._app.get_window())
-        
+            app_window = self._app.get_window()
+            widget_instance.gtk_widget.set_transient_for(app_window)
+
     def _delete_execute(self):
-        from gazpacho.placeholder import Placeholder
+        from gazpacho import placeholder
+        widget_instance = self._widget_instance
+        
         if self._parent:
             if self._placeholder is None:
-                self._placeholder = Placeholder(self._app)
-            self._widget.replace(self._widget.widget, self._placeholder,
-                                 self._parent)
+                self._placeholder = placeholder.Placeholder(self._app)
 
-        self._widget.widget.hide()
-        self._widget.project.remove_widget(self._widget.widget)
+            widget.replace_widget(widget_instance.gtk_widget,
+                                  self._placeholder, self._parent)
 
+            widget_klass = widget_instance.klass
+            self._parent.set_default_gtk_packing_properties(self._placeholder,
+                                                            widget_klass)
+        
+        widget_instance.gtk_widget.hide()
+        widget_instance.project.remove_widget(widget_instance.gtk_widget)
+
+        # if we are removing a toolbar or a menubar we need to update the
+        # uimanager
+        if isinstance(widget_instance.gtk_widget, (gtk.Toolbar, gtk.MenuBar)):
+            d = widget_instance.project.uim.get_ui(widget_instance)
+            self._ui_defs = dict(d)
+            widget_instance.project.uim.remove_widget(widget_instance)
+        
         return True
 
     def execute(self):
@@ -382,18 +413,17 @@ class CommandDeletePlaceholder(Command):
         self._parent = parent
         self._create = False
 
-        children = self._parent.widget.get_children()
+        children = self._parent.gtk_widget.get_children()
         self._pos = children.index(placeholder)
         
     def _create_execute(self):
-        from gazpacho.placeholder import Placeholder
+        from gazpacho import placeholder
         # create a placeholder and insert it at self._pos
-        self._placeholder = Placeholder(self._app)
-        self._parent.widget.add(self._placeholder)
-        self._parent.widget.reorder_child(self._placeholder, self._pos)
+        self._placeholder = placeholder.Placeholder(self._app)
+        self._parent.gtk_widget.add(self._placeholder)
+        self._parent.gtk_widget.reorder_child(self._placeholder, self._pos)
 
     def _delete_execute(self):
-        from gazpacho.placeholder import Placeholder
         self._placeholder.destroy()
         self._placeholder = None
         return True
@@ -419,14 +449,13 @@ class CommandInsertPlaceholder(Command):
         
 
     def _insert_execute(self):
-        from gazpacho.placeholder import Placeholder
+        from gazpacho import placeholder
         # create a placeholder and insert it at self._pos
-        self._placeholder = Placeholder(self._app)
-        self._box.widget.add(self._placeholder)
-        self._box.widget.reorder_child(self._placeholder, self._pos)
+        self._placeholder = placeholder.Placeholder(self._app)
+        self._box.gtk_widget.add(self._placeholder)
+        self._box.gtk_widget.reorder_child(self._placeholder, self._pos)
 
     def _delete_execute(self):
-        from gazpacho.placeholder import Placeholder
         self._placeholder.destroy()
         self._placeholder = None
         return True
@@ -450,10 +479,34 @@ class CommandSetProperty(Command):
         self._value = value
 
     def execute(self):
+        # we have to be careful with tbe name property
+        if self._property.klass.id == 'name':
+            # make sure the name is unique
+            project = self._property.widget.project
+            other = project.get_widget_by_name(self._value)
+            if other:
+                msg = _('There is another widget with the name "%s" in this '
+                        'project' % self._value)
+                self._app.show_message(msg)
+                self._app.refresh_editor()
+                return False
+
+            # check the name is not empty
+            if self._value == '':
+                msg = _("The name can not be empty")
+                self._app.show_message(msg)
+                self._app.refresh_editor()
+                return False
+            
         new_value = self._value
         # store the current value for undo
         self._value = self._property.value
         self._property.set(new_value)
+        # if the property is the name, we explicitily set the name of the
+        # gwidget to trigger the notify signal so several parts of the
+        # interface get updated
+        if self._property.klass.id == 'name':
+            self._property.widget.name = new_value
 
         return True
 
@@ -505,120 +558,103 @@ class CommandSetTranslatableProperty(Command):
     def collapse(self, other):
         return False
 
-class CommandSetName(Command):
-    def __init__(self, widget=None, old_name=None, name=None,
-                 description=None, application=None):
-        Command.__init__(self, description, application)
-        self._widget = widget
-        self._old_name = old_name
-        self._name = name
-
-    def execute(self):
-        self._widget.name = self._name
-        self._old_name, self._name = self._name, self._old_name
-        return True
-
-    def unifies(self, other):
-        if isinstance(other, CommandSetName):
-            return self._widget == other._widget
-        return False
-
-    def collapse(self, other):
-        if not isinstance(other, CommandSetName):
-            return
-
-        self._old_name = other._old_name
-        other._old_name = None
-        self._description = _('Rename %s to %s') % (self._name,
-                                                    self._old_name)
-
-        self._app.refresh_undo_and_redo()
-        
 class CommandAddRemoveSignal(Command):
-    def __init__(self, add=True, signal=None, widget=None,
+    def __init__(self, add=True, signal=None, widget_instance=None,
                  description=None, application=None):
         Command.__init__(self, description, application)
         self._add = add
         self._signal = signal
-        self._widget = widget
+        self._widget_instance = widget_instance
 
     def execute(self):
         if self._add:
-            self._widget.add_signal_handler(self._signal)
+            self._widget_instance.add_signal_handler(self._signal)
         else:
-            self._widget.remove_signal_handler(self._signal)
+            self._widget_instance.remove_signal_handler(self._signal)
 
         self._add = not self._add
         return True
 
-class CommandCopyCutPaste(Command):
-    COPY, CUT, PASTE = range(3)
+class CommandCutPaste(Command):
+
+    CUT, PASTE = range(2)
     
-    def __init__(self, widget, placeholder, operation,
+    def __init__(self, widget_instance, project, placeholder, operation,
                  description=None, application=None):
         Command.__init__(self, description, application)
-        
-        self._widget = widget
+
+        self._project = project
         self._placeholder = placeholder
         self._operation = operation
-        self._clipboard = application.get_clipboard()
-        self._undo = False
-        
-    def execute(self):
-        if self._operation == CommandCopyCutPaste.COPY:
-            self._execute_copy()
-        elif self._operation == CommandCopyCutPaste.CUT:
-            if self._undo:
-                self._execute_paste()
-            else:
-                self._execute_cut()
-        elif self._operation == CommandCopyCutPaste.PASTE:
-            if self._undo:
-                self._execute_cut()
-            else:
-                self._execute_paste()
+        self._widget_instance = widget_instance
+        self._ui_defs = {}
 
-        self._undo = not self._undo
-        
-    def _execute_copy(self):
-        if self._undo:
-            self._clipboard.remove_widget(self._widget)
+    def execute(self):
+        if self._operation == CommandCutPaste.CUT:
+            self._execute_cut()
+            self._operation = CommandCutPaste.PASTE
         else:
-            self._clipboard.add_widget(self._widget)
-    
+            self._execute_paste()
+            self._operation = CommandCutPaste.CUT
+        
     def _execute_cut(self):
-        from gazpacho.placeholder import Placeholder
-        self._clipboard.add_widget(self._widget)
-        parent = self._widget.get_parent()
-        if parent is not None:
-            if self._placeholder is None:
-                self._placeholder = Placeholder(self._app)
-            self._widget.replace(self._widget.widget, self._placeholder, parent)
+        from gazpacho import placeholder
+        widget_instance = self._widget_instance
         
-        self._widget.widget.hide()
-        self._widget.project.remove_widget(self._widget.widget)
-    
+        if not widget_instance.is_toplevel():
+            parent = widget_instance.get_parent()
+
+            if not self._placeholder:
+                self._placeholder = placeholder.Placeholder(self._app)
+
+            widget.replace_widget(widget_instance.gtk_widget,
+                                  self._placeholder, parent)
+            parent.set_default_gtk_packing_properties(self._placeholder,
+                                                      widget_instance.klass)
+
+        widget_instance.gtk_widget.hide()
+        widget_instance.project.remove_widget(widget_instance.gtk_widget)
+
+        # if we are removing a toolbar or a menubar we need to update the
+        # uimanager
+        if isinstance(widget_instance.gtk_widget, (gtk.Toolbar, gtk.MenuBar)):
+            d = widget_instance.project.uim.get_ui(widget_instance)
+            self._ui_defs = dict(d)
+            widget_instance.project.uim.remove_widget(widget_instance)
+
     def _execute_paste(self):
-        parent = get_parent(self._placeholder)
-        project = parent.project
-        
-        if not self._widget.is_toplevel():
-            self._widget.replace(self._placeholder, self._widget.widget, parent)
+        # if we did cut a toolbar/menubar the uim removed
+        # its ui definitions so we need to add them again
+        if len(self._ui_defs.keys()) > 0:
+            self._widget_instance.project.uim.add_widget(self._widget_instance,
+                                                         self._ui_defs)
+            self._ui_defs = {}
+
+        if self._widget_instance.is_toplevel():
+            project = self._project
+        else:
+            parent = util.get_parent(self._placeholder)
+            project = parent.project
+            widget.replace_widget(self._placeholder,
+                                  self._widget_instance.gtk_widget,
+                                  parent)
             
-        project.add_widget(self._widget.widget)
-        project.selection_set(self._widget.widget, True)
+        project.add_widget(self._widget_instance.gtk_widget)
+        project.selection_set(self._widget_instance.gtk_widget, True)
         
-        self._widget.widget.show_all()
-        
-        self._clipboard.remove_widget(self._widget)
+        self._widget_instance.gtk_widget.show_all()
+
+        # We need to store the project of a toplevel widget to use
+        # when undoing the cut.
+        self._project = project
 
 class CommandAddRemoveAction(Command):
-    def __init__(self, parent, gaction, add=True,
+    def __init__(self, parent, gact, add=True,
                  description=None, application=None):
         Command.__init__(self, description, application)
 
         self.add = add
-        self.gaction = gaction
+        self.gact = gact
         self.parent = parent
         
     def execute(self):
@@ -630,35 +666,35 @@ class CommandAddRemoveAction(Command):
         self.add = not self.add
     
     def _add_execute(self):
-        self.parent.append(self.gaction)
+        self.parent.append(self.gact)
     
     def _remove_execute(self):
-        self.parent.remove(self.gaction)
+        self.parent.remove(self.gact)
 
 class CommandEditAction(Command):
-    def __init__(self, gaction, new_values, description=None,
+    def __init__(self, gact, new_values, description=None,
                  application=None):
         Command.__init__(self, description, application)
         
         self.new_values = new_values
-        self.gaction = gaction
+        self.gact = gact
 
     def execute(self):
         old_values = {
-            'name' : self.gaction.name,
-            'label': self.gaction.label,
-            'stock_id': self.gaction.stock_id,
-            'tooltip': self.gaction.tooltip,
-            'accelerator': self.gaction.accelerator,
-            'callback': self.gaction.callback,
+            'name' : self.gact.name,
+            'label': self.gact.label,
+            'stock_id': self.gact.stock_id,
+            'tooltip': self.gact.tooltip,
+            'accelerator': self.gact.accelerator,
+            'callback': self.gact.callback,
             }
-        self.gaction.name = self.new_values['name']
-        self.gaction.label = self.new_values['label']
-        self.gaction.stock_id = self.new_values['stock_id']
-        self.gaction.tooltip = self.new_values['tooltip']
-        self.gaction.accelerator = self.new_values['accelerator']
-        self.gaction.callback = self.new_values['callback']
-        self.gaction.parent.update_action(self.gaction, old_values['name'])
+        self.gact.name = self.new_values['name']
+        self.gact.label = self.new_values['label']
+        self.gact.stock_id = self.new_values['stock_id']
+        self.gact.tooltip = self.new_values['tooltip']
+        self.gact.accelerator = self.new_values['accelerator']
+        self.gact.callback = self.new_values['callback']
+        self.gact.parent.update_action(self.gact, old_values['name'])
         self.new_values = old_values
 
 class CommandAddRemoveActionGroup(Command):
