@@ -1,4 +1,4 @@
-# Copyright (C) 2004,2005 by SICEm S.L. and Imendio
+# Copyright (C) 2004,2005 by SICEm S.L. and Imendio AB
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public License
@@ -20,7 +20,6 @@ from gazpacho import parameter
 from gazpacho import propertyclass as pc
 from gazpacho.loader import tags
 from gazpacho.signaleditor import SignalEditor
-from gazpacho.l10n import _
 
 TABLE_ROW_SPACING = 2
 TABLE_COLUMN_SPACING = 6
@@ -60,6 +59,17 @@ class EditorProperty:
         # Should be overridden
         return
 
+    def set_property(self, value):
+        self._app.command_manager.set_property(self.property, value)
+
+    def set_translatable_property(self, text, comment_text, is_translatable,
+                                  has_context):
+        self._app.command_manager.set_translatable_property(self.property,
+                                                            text,
+                                                            comment_text,
+                                                            is_translatable,
+                                                            has_context)
+        
     def _start_load(self, widget):
         self.property = widget.get_glade_property(self.klass.id)
         self.property.loading = True
@@ -67,6 +77,12 @@ class EditorProperty:
     def _finish_load(self):
         self.property.loading = False
 
+    def _internal_load(self, value):
+        """This is the virtual method that actually put the data in the editor
+        widget.
+        Most subclasses implements this (But not EditorPropertySpecial)
+        """
+    
 class EditorPropertyNumeric(EditorProperty):
 
     def __init__(self, property_class, app):
@@ -75,38 +91,35 @@ class EditorPropertyNumeric(EditorProperty):
     def load(self, widget):
         self._start_load(widget)
         
+        self._internal_load(self.property.value)
+
+        self.input.set_data('user_data', self)
+        self._finish_load()
+
+    def _internal_load(self, value):
         if self.klass.id in pc.UNICHAR_PROPERTIES:
-            self._load_unichar()
-            return
-        
-        if self.klass.optional:
+            if isinstance(self.input, gtk.Editable):
+                self.input.delete_text(0, -1)
+                self.input.insert_text(value)
+        elif self.klass.optional:
             button, spin = self.input.get_children()
             button.set_active(self.property.enabled)
             spin.set_sensitive(self.property.enabled)
             button.set_data('user_data', self)
+            v = float(value)
+            spin.set_value(v)
         else:
-            spin = self.input
-
-        v = float(self.property.value)
-
-        spin.set_value(v)
-        self.input.set_data('user_data', self)
-        self._finish_load()
-
-    def _load_unichar(self):
-        if isinstance(self.input, gtk.Editable):
-            value = self.property.value
-            self.input.delete_text(0, -1)
-            self.input.insert_text(value)
-        self.input.set_data('user_data', self)
-
+            v = float(value)
+            self.input.set_value(v)
+                
     def _changed_unichar(self, entry):
         if self.property.loading:
             return
         
         text = entry.get_chars(0, -1)
         unich = text[0]
-        self._app.command_manager.set_property(self.property, unich)
+
+        self.set_property(unich)
         
     def _create_input_unichar(self):
         entry = gtk.Entry()
@@ -128,8 +141,8 @@ class EditorPropertyNumeric(EditorProperty):
             value = spin.get_value()
         else:
             value = 0
-            
-        self._app.command_manager.set_property(self.property, value)
+
+        self.set_property(value)
 
     def _changed_enabled(self, check_button):
         if self.property.loading:
@@ -188,7 +201,7 @@ class EditorPropertyText(EditorProperty):
 
     def __init__(self, property_class, app):
         EditorProperty.__init__(self, property_class, app)
-        
+
         if self.klass.is_translatable:
             i18n_button = gtk.Button('...')
             i18n_button.connect('clicked', self._show_i18n_dialog)
@@ -222,31 +235,33 @@ class EditorPropertyText(EditorProperty):
         
     def load(self, widget):
         self._start_load(widget)
-        text = self.property.value
 
+        self._internal_load(self.property.value)
+
+        self.input.set_data('user_data', self)
+
+        self._finish_load()
+
+    def _internal_load(self, value):
         # Text inputs are an entry or text view inside an hbox
         text_widget = self.input.get_children()[0]
         
         if isinstance(text_widget, gtk.Editable):
             pos = text_widget.get_position()
             text_widget.delete_text(0, -1)
-            if text:
-                text_widget.insert_text(text)
+            if value:
+                text_widget.insert_text(value)
 
             text_widget.set_position(pos)
 
         elif isinstance(text_widget, gtk.TextView):
             text_buffer = text_widget.get_buffer()
-            text_buffer.set_text(text)
+            text_buffer.set_text(value)
         else:
             print _('Invalid Text Widget type')
 
-        self.input.set_data('user_data', self)
-
-        self._finish_load()
-
     def _changed_text_common(self, text):
-        self._app.command_manager.set_property(self.property, text)
+        self.set_property(text)
 
     def _changed_text(self, entry):
         if self.property.loading:
@@ -285,7 +300,8 @@ class EditorPropertyText(EditorProperty):
         dialog.vbox.pack_start(vbox, True, True, 0)
 
         # The text content
-        label = gtk.Label(_('Text:'))
+        label = gtk.Label()
+        label.set_markup_with_mnemonic(_('_Text:'))
         label.show()
         label.set_alignment(0, 0.5)
         vbox.pack_start(label, False, False, 0)
@@ -300,6 +316,7 @@ class EditorPropertyText(EditorProperty):
 
         text_view = gtk.TextView()
         text_view.show()
+        label.set_mnemonic_widget(text_view)
 
         scrolled_window.add(text_view)
 
@@ -315,18 +332,19 @@ class EditorPropertyText(EditorProperty):
 
         vbox.pack_start (hbox, False, False, 0)
 
-        translatable_button = gtk.CheckButton (_("Translatable"))
+        translatable_button = gtk.CheckButton (_("T_ranslatable"))
         translatable_button.set_active (is_translatable)
         translatable_button.show()
         hbox.pack_start(translatable_button, False, False, 0)
 
-        context_button = gtk.CheckButton (_("Has context prefix"))
+        context_button = gtk.CheckButton (_("_Has context prefix"))
         context_button.set_active (has_context)
         context_button.show()
         hbox.pack_start(context_button, False, False, 0)
 
         # Comments
-        label = gtk.Label(_("Comments for translators:"))
+        label = gtk.Label()
+        label.set_markup_with_mnemonic(_("Co_mments for translators:"))
         label.show()
         label.set_alignment(0, 0.5)
         vbox.pack_start (label, False, False, 0)
@@ -341,6 +359,7 @@ class EditorPropertyText(EditorProperty):
 
         comment_view = gtk.TextView()
         comment_view.show()
+        label.set_mnemonic_widget(comment_view)
 
         scrolled_window.add(comment_view)
 
@@ -360,16 +379,11 @@ class EditorPropertyText(EditorProperty):
             (start, end) = comment_buffer.get_bounds()
             comment_text = comment_buffer.get_text(start, end, False)
 
-            self._app.command_manager.set_translatable_property(self.property,
-                                                                text,
-                                                                comment_text,
-                                                                is_translatable,
-                                                                has_context)
+            self.set_translatable_property(text, comment_text, is_translatable,
+                                           has_context)
 
             # Update the entry in the property editor
-            self.loading = True
-            self._load_text()
-            self.loading = False
+            self.load(self.property.widget)
 
         dialog.destroy()
 
@@ -391,14 +405,17 @@ class EditorPropertyEnum(EditorProperty):
 
     def load(self, widget):
         self._start_load(widget)
+        self._internal_load(self.property.value)
+        self._finish_load()
+
+    def _internal_load(self, value):
         idx = 0
         for choice in self.property.klass.choices:
-            if choice['value'] == self.property.value:
+            if choice['value'] == value:
                 break
             idx += 1
         self.input.set_active(idx)
-        self._finish_load()
-
+        
     def _changed_enum(self, combo_box):
         if self.property.loading:
             return
@@ -406,7 +423,7 @@ class EditorPropertyEnum(EditorProperty):
         active_iter = combo_box.get_active_iter()
         model = combo_box.get_model()
         choice = model.get_value(active_iter, 1)
-        self._app.command_manager.set_property(self.property, choice['value'])
+        self.set_property(choice['value'])
    
 class EditorPropertyFlags(EditorProperty):
     def __init__(self, property_class, app):
@@ -428,14 +445,13 @@ class EditorPropertyFlags(EditorProperty):
 
     def load(self, widget):
         self._start_load(widget)
-        self._load_flags()
+        self._internal_load(self.property.value)
         self._finish_load()
 
-    def _load_flags(self):
+    def _internal_load(self, value):
         # the entry should be the first child
         entry = self.input.get_children()[0]
         text = ''
-        value = self.property.value
         flags_class = self.klass.spec.flags_class
         
         # Populate the model with the flags
@@ -511,10 +527,9 @@ class EditorPropertyFlags(EditorProperty):
             # if the new value is different from the old value, we need to
             # update the property
             if new_value != value:
-                self._app.command_manager.set_property(self.property,
-                                                       new_value)
+                self.set_property(new_value)
                 # update the entry in the property editor
-                self._load_flags()
+                self._internal_load(new_value)
                 
         dialog.destroy()
 
@@ -529,11 +544,15 @@ class EditorPropertyBoolean(EditorProperty):
 
     def load(self, widget):
         self._start_load(widget)
-        label = self.input.get_child()
-        if not isinstance(label, gtk.Label):
-            return
 
-        state = self.property.value
+        self._internal_load(self.property.value)
+        
+        self.input.set_data('user_data', self)
+        self._finish_load()
+
+    def _internal_load(self, state):
+        label = self.input.get_child()
+
         if state:
             value = tags.YES
         else:
@@ -541,9 +560,7 @@ class EditorPropertyBoolean(EditorProperty):
         label.set_text(value)
             
         self.input.set_active(state)
-        self.input.set_data('user_data', self)
-        self._finish_load()
-
+        
     def _changed_boolean(self, button):
         if self.property.loading:
             return
@@ -555,8 +572,7 @@ class EditorPropertyBoolean(EditorProperty):
             
         label = button.get_child()
         label.set_text(value)
-        self._app.command_manager.set_property(self.property,
-                                               button.get_active())
+        self.set_property(button.get_active())
 
 class IncompleteFunctions(Exception): pass
 
@@ -567,10 +583,11 @@ class EditorPropertySpecial(EditorProperty):
         if self.klass._editor_create_function is None:
             raise IncompleteFunctions(_("EditorPropertySpecial without create function: %s") % self.klass.id)
 
-        if self.klass._editor_load_function is None:
-            raise IncompleteFunctions(_("EditorPropertySpecial without load function: %s") % self.klass.id)
-        
-        self.input = self.klass._editor_create_function()
+        if self.klass._editor_update_function is None:
+            raise IncompleteFunctions(_("EditorPropertySpecial without update function: %s") % self.klass.id)
+
+        context = app.get_current_context()
+        self.input = self.klass._editor_create_function(context)
 
     def load(self, widget):
         self._start_load(widget)
@@ -580,14 +597,16 @@ class EditorPropertySpecial(EditorProperty):
         proxy.connect('value-set', self._value_changed, widget)
         proxy.connect('property-set', self._property_set, widget)
 
-        self.klass._editor_load_function(self.input, widget.gtk_widget, proxy)
+        context = widget.project.context
+        self.klass._editor_update_function(context, self.input,
+                                           widget.gtk_widget, proxy)
         self._finish_load()
 
     def _value_changed(self, proxy, value, widget):
         if self.property.loading:
             return
-        
-        self._app.command_manager.set_property(self.property, value)
+
+        self.set_property(value)
 
     def _property_set(self, proxy, prop_id, value, widget):
         property = widget.get_glade_property(prop_id)
@@ -595,7 +614,7 @@ class EditorPropertySpecial(EditorProperty):
         if not property or property.loading: 
             return
 
-        self._app.command_manager.set_property(property, value)
+        self.set_property(value)
 
 class EditorValueProxy(gobject.GObject):
     __gsignals__ = {
@@ -629,10 +648,10 @@ def create_editor_property(property_class, app):
     if property_class._editor_create_function:
         return EditorPropertySpecial(property_class, app)
 
-    try:
+    if property_map.has_key(property_class.type):
         return property_map[property_class.type](property_class, app)
-    except KeyError:
-        print "Unsupported type for property class %s." % (property_class.name)
+    else:
+        return None
 
 
 class EditorTable:
@@ -686,8 +705,8 @@ class EditorTable:
 
     def _create_item_label(self, property_class):
         label = gtk.Label("%s:" % property_class.name)
-        label.set_size_request(140, -1)
-        label.set_alignment(0.0, 0.5)
+        label.set_size_request(-1, -1)
+        label.set_alignment(1.0, 0.5)
         
         # we need to wrap the label in an event box to add tooltips
         eventbox = gtk.EventBox()
@@ -700,7 +719,7 @@ class EditorTable:
     def _append_item(self, property_class):
         prop = create_editor_property(property_class, self.editor._app)
 
-        if prop.input:
+        if prop and prop.input:
             label = self._create_item_label(property_class)
             self._attach_row(label, prop.input, self.rows)
             self.rows += 1
@@ -708,19 +727,11 @@ class EditorTable:
         return prop
     
     def append_standard_fields(self):
-        # Name
-        label = gtk.Label(_('Name') + ':'); label.set_alignment(0.0, 0.5)
-        self.name_entry = gtk.Entry()
-        self.name_entry.connect('changed', self.editor._widget_name_changed)
-        self._attach_row(label, self.name_entry, self.rows)
-        self.rows += 1
-
         # Class
-        label = gtk.Label(_('Class')+':'); label.set_alignment(0.0, 0.5)
-        entry = gtk.Entry()
-        entry.set_text(self.widget_klass.name)
-        entry.set_editable(False)
-        self._attach_row(label, entry, self.rows)
+        label = gtk.Label(_('Class')+':'); label.set_alignment(1.0, 0.5)
+        class_label = gtk.Label(self.widget_klass.name)
+        class_label.set_alignment(0.0, 0.5)
+        self._attach_row(label, class_label, self.rows)
         self.rows += 1
 
     def append_items(self, klass, thelist, common):
@@ -732,6 +743,15 @@ class EditorTable:
             if common != property_class.common:
                 continue
 
+            if not property_class.editable:
+                continue
+            
+            # don't add properties with type == object unless it has a custom
+            # editor
+            if property_class.type == gobject.TYPE_OBJECT and \
+                   not property_class._editor_create_function:
+                continue
+            
             prop = self._append_item(property_class)
             if prop:
                 thelist.insert(0, prop)
@@ -808,18 +828,6 @@ class Editor(gtk.Notebook):
     def do_add_signal(self, id_widget, type_widget, id_signal, callback_name):
         pass
 
-    def update_widget_name(self):
-        pass # XXX TODO
-
-    def _widget_name_changed(self, editable):
-        """ We call this function when the user changes the widget name using
-        the entry on the properties editor. """
-        if self._loading: return
-
-        widget = self._loaded_widget
-        new_name = editable.get_chars(0, -1)
-        self._app.command_manager.set_name(widget, new_name)
-
     def _table_create(self, klass, common):
         table = EditorTable(self, klass, common)
         if not common:
@@ -845,7 +853,7 @@ class Editor(gtk.Notebook):
     def _load_widget_page(self, klass):
         # Remove the old table that was in this container
         children = self._vbox_widget.get_children()
-        [self._vbox_widget.remove(child) for child in children]
+        map(self._vbox_widget.remove, children)
 
         if not klass:
             return
@@ -858,7 +866,7 @@ class Editor(gtk.Notebook):
     def _load_common_page(self, klass):
         # Remove the old table that was in this container
         children = self._vbox_common.get_children()
-        [self._vbox_common.remove(child) for child in children]
+        map(self._vbox_common.remove, children)
 
         if not klass:
             return
@@ -929,7 +937,6 @@ class Editor(gtk.Notebook):
 
         # Load each EditorProperty
         table = self._get_table_from_klass(klass, False)
-        table.name_entry.set_text(widget.name)
         for widget_property in table.properties:
             widget_property.load(widget)
 
